@@ -36,9 +36,7 @@
 
 #define O_DEBUG 1
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #ifdef __GNUC__
 # define alloca __builtin_alloca
@@ -96,45 +94,101 @@ static atom_t ATOM_max_message_size;    /* "message_size" */
 static atom_t ATOM_file_no;		/* "file_no" */
 static atom_t ATOM_ip_add_membership;	/* "ip_add_membership" */
 static atom_t ATOM_ip_drop_membership;	/* "ip_drop_membership" */
+static atom_t ATOM_sndbuf;	        /* "sndbuf" */
 static functor_t FUNCTOR_socket1;	/* $socket(Id) */
 
+static int get_socket_from_stream(term_t t, IOSTREAM **s, nbio_sock_t *sp);
+
+
+		 /*******************************
+		 *	      SYMBOL		*
+		 *******************************/
+
+static void
+acquire_socket_symbol(atom_t symbol)
+{ nbio_sock_t s = *(nbio_sock_t*)PL_blob_data(symbol, NULL, NULL);
+
+  nbio_set_symbol(s, symbol);
+}
+
+static int
+release_socket_symbol(atom_t symbol)
+{ nbio_sock_t s = *(nbio_sock_t*)PL_blob_data(symbol, NULL, NULL);
+
+  freeSocket(s);
+  return TRUE;
+}
+
+static int
+compare_socket_symbols(atom_t a, atom_t b)
+{ nbio_sock_t sa = *(nbio_sock_t*)PL_blob_data(a, NULL, NULL);
+  nbio_sock_t sb = *(nbio_sock_t*)PL_blob_data(b, NULL, NULL);
+
+  return ( sa > sb ?  1 :
+	   sa < sb ? -1 : 0
+	 );
+}
+
+
+static int
+write_socket_symbol(IOSTREAM *s, atom_t symbol, int flags)
+{ nbio_sock_t sock = *(nbio_sock_t*)PL_blob_data(symbol, NULL, NULL);
+
+  Sfprintf(s, "<socket>(%p)", sock);
+  return TRUE;
+}
+
+
+static PL_blob_t socket_blob =
+{ PL_BLOB_MAGIC,
+  0,
+  "socket",
+  release_socket_symbol,
+  compare_socket_symbols,
+  write_socket_symbol,
+  acquire_socket_symbol
+};
+
+
+static int
+tcp_unify_socket(term_t handle, nbio_sock_t socket)
+{ if ( PL_unify_blob(handle, &socket, sizeof(socket), &socket_blob) )
+    return TRUE;
+
+  if ( !PL_is_variable(handle) )
+    return PL_uninstantiation_error(handle);
+
+  return FALSE;					/* (resource) error */
+}
+
+
+static int
+tcp_get_socket(term_t handle, nbio_sock_t *sp)
+{ PL_blob_t *type;
+  void *data;
+
+  if ( PL_get_blob(handle, &data, NULL, &type) && type == &socket_blob)
+  { nbio_sock_t s = *(nbio_sock_t*)data;
+
+    if ( !is_nbio_socket(s) )
+      return PL_existence_error("socket", handle);
+
+    *sp = s;
+
+    return TRUE;
+  }
+
+  if ( get_socket_from_stream(handle, NULL, sp) )
+    return TRUE;
+
+  return PL_type_error("socket", handle);
+}
+
 
 		 /*******************************
 		 *	     CONVERSION		*
 		 *******************************/
 
-NBIO_EXPORT(int)
-tcp_get_socket(term_t Socket, int *id)
-{ IOSTREAM *s;
-  int socket;
-
-  if ( PL_is_functor(Socket, FUNCTOR_socket1) )
-  { term_t a = PL_new_term_ref();
-
-    _PL_get_arg(1, Socket, a);
-    if ( PL_get_integer(a, id) )
-      return TRUE;
-  }
-
-  if ( PL_get_stream_handle(Socket, &s) )
-  { socket = (int)(intptr_t)s->handle;
-
-    *id = socket;
-    return TRUE;
-  }
-
-  return pl_error(NULL, 0, NULL, ERR_ARGTYPE, -1, Socket, "socket");
-}
-
-
-static int
-tcp_unify_socket(term_t Socket, int id)
-{ return PL_unify_term(Socket,
-		       PL_FUNCTOR, FUNCTOR_socket1,
-		         IntArg(id));
-}
-
-
 static foreign_t
 pl_host_to_address(term_t Host, term_t Ip)
 { struct in_addr ip;
@@ -143,11 +197,12 @@ pl_host_to_address(term_t Host, term_t Ip)
   if ( PL_get_atom_chars(Host, &host_name) )
   { struct addrinfo hints;
     struct addrinfo *res;
+    int rc;
 
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = AF_INET;
 
-    if ( getaddrinfo(host_name, NULL, &hints, &res) == 0 )
+    if ( (rc=getaddrinfo(host_name, NULL, &hints, &res)) == 0 )
     { int rc;
 
       switch( res->ai_family )
@@ -170,7 +225,7 @@ pl_host_to_address(term_t Host, term_t Ip)
 
       return rc;
     } else
-    { return nbio_error(GET_H_ERRNO, TCP_HERRNO);
+    { return nbio_error(rc, TCP_GAI_ERRNO);
     }
   } else if ( nbio_get_ip(Ip, &ip) )
   { struct hostent *host;
@@ -193,7 +248,7 @@ pl_host_to_address(term_t Host, term_t Ip)
 
 static foreign_t
 pl_setopt(term_t Socket, term_t opt)
-{ int socket;
+{ nbio_sock_t socket;
   atom_t a;
   size_t arity;
 
@@ -288,10 +343,23 @@ pl_setopt(term_t Socket, term_t opt)
 
       if ( setsockopt(nbio_fd(socket), IPPROTO_IP, opname,
 		      (void*)&mreq, sizeof(mreq)) < 0 )
-	return nbio_error(GET_H_ERRNO, TCP_HERRNO);
+	return nbio_error(GET_ERRNO, TCP_ERRNO);
       else
 	return TRUE;
 #endif
+    } else if ( a == ATOM_sndbuf && arity == 1 )
+    { int bufsize;
+      int rc;
+      term_t a = PL_new_term_ref();
+      _PL_get_arg(1, opt, a);
+      if ( !PL_get_integer(a, &bufsize) )
+	return pl_error(NULL, 0, NULL, ERR_DOMAIN, a, "integer");
+      if ( (rc=nbio_setopt(socket, TCP_SNDBUF, bufsize) == 0) )
+	return TRUE;
+      if ( rc == -2 )
+	goto not_implemented;
+
+      return FALSE;
     }
   }
 
@@ -302,7 +370,7 @@ not_implemented:
 
 static foreign_t
 pl_getopt(term_t Socket, term_t opt)
-{ int socket;
+{ nbio_sock_t socket;
   atom_t a;
   size_t arity;
 
@@ -364,7 +432,7 @@ udp_receive(term_t Socket, term_t Data, term_t From, term_t options)
 #else
   socklen_t alen = sizeof(sockaddr);
 #endif
-  int socket;
+  nbio_sock_t socket;
   int flags = 0;
   char smallbuf[UDP_DEFAULT_BUFSIZE];
   char *buf = smallbuf;
@@ -448,7 +516,7 @@ udp_send(term_t Socket, term_t Data, term_t To, term_t Options)
 #else
   int alen = sizeof(sockaddr);
 #endif
-  int socket;
+  nbio_sock_t socket;
   int flags = 0L;
   char *data;
   size_t dlen;
@@ -477,10 +545,9 @@ udp_send(term_t Socket, term_t Data, term_t To, term_t Options)
 
 static foreign_t
 create_socket(term_t socket, int type)
-{ int sock;
+{ nbio_sock_t sock;
 
-  sock = nbio_socket(AF_INET, type, 0);
-  if ( sock < 0 )
+  if ( !(sock = nbio_socket(AF_INET, type, 0)) )
     return FALSE;
 
   return tcp_unify_socket(socket, sock);
@@ -501,7 +568,7 @@ udp_socket(term_t socket)
 
 static foreign_t
 pl_connect(term_t Socket, term_t Address)
-{ int sock;
+{ nbio_sock_t sock;
   struct sockaddr_in sockaddr;
 
   if ( !tcp_get_socket(Socket, &sock) ||
@@ -518,7 +585,7 @@ pl_connect(term_t Socket, term_t Address)
 static foreign_t
 pl_bind(term_t Socket, term_t Address)
 { struct sockaddr_in sockaddr;
-  int socket;
+  nbio_sock_t socket;
   term_t varport = 0;
 
   memset(&sockaddr, 0, sizeof(sockaddr));
@@ -550,14 +617,14 @@ pl_bind(term_t Socket, term_t Address)
 
 static foreign_t
 pl_accept(term_t Master, term_t Slave, term_t Peer)
-{ int master, slave;
+{ nbio_sock_t master, slave;
   struct sockaddr_in addr;
   socklen_t addrlen = sizeof(addr);
 
   if ( !tcp_get_socket(Master, &master) )
     return FALSE;
 
-  if ( (slave = nbio_accept(master, (struct sockaddr*)&addr, &addrlen)) < 0 )
+  if ( !(slave = nbio_accept(master, (struct sockaddr*)&addr, &addrlen)) )
     return FALSE;
 					/* TBD: close on failure */
   if ( nbio_unify_ip4(Peer, ntohl(addr.sin_addr.s_addr)) &&
@@ -590,7 +657,7 @@ pl_gethostname(term_t name)
 	hname = PL_new_atom(buf);
 
     } else
-    { return nbio_error(GET_H_ERRNO, TCP_HERRNO);
+    { return nbio_error(GET_ERRNO, TCP_ERRNO);
     }
   }
 
@@ -598,172 +665,6 @@ pl_gethostname(term_t name)
 }
 
 
-#ifdef __WINDOWS__
-
-		 /*******************************
-		 *	       SELECT		*
-		 *******************************/
-
-typedef struct fdentry
-{ int fd;
-  term_t stream;
-  struct fdentry *next;
-} fdentry;
-
-
-static term_t
-findmap(fdentry *map, int fd)
-{ for( ; map; map = map->next )
-  { if ( map->fd == fd )
-      return map->stream;
-  }
-  assert(0);
-  return 0;
-}
-
-
-static int
-is_socket_stream(IOSTREAM *s)
-{ if ( s->functions == &readFunctions /* ||
-       s->functions == &writeFunctions */ )
-    return TRUE;
-
-  return FALSE;
-}
-
-
-#ifndef SIO_GETPENDING
-static size_t
-Spending(IOSTREAM *s)
-{ if ( s->bufp < s->limitp )
-    return s->limitp - s->bufp;
-  return 0;
-}
-#endif
-
-static foreign_t
-tcp_select(term_t Streams, term_t Available, term_t timeout)
-{ fd_set fds;
-  struct timeval t, *to;
-  double time;
-  int max = 0, ret, min = 1000000;
-  fdentry *map     = NULL;
-  term_t head      = PL_new_term_ref();
-  term_t streams   = PL_copy_term_ref(Streams);
-  term_t available = PL_copy_term_ref(Available);
-  term_t ahead     = PL_new_term_ref();
-  int from_buffer  = 0;
-  atom_t a;
-#ifdef __WINDOWS__
-  int count = 0;			/* on Windows the setsize is limited */
-#endif					/* to FD_SETSIZE */
-
-  FD_ZERO(&fds);
-  while( PL_get_list(streams, head, streams) )
-  { IOSTREAM *s;
-    nbio_sock_t fd;
-    fdentry *e;
-
-    if ( !PL_get_stream_handle(head, &s) )
-      return FALSE;
-
-    fd = fdFromHandle(s->handle);
-
-    PL_release_stream(s);
-    if ( fd < 0 || !is_socket_stream(s) )
-    { return pl_error("tcp_select", 3, NULL, ERR_DOMAIN,
-		      head, "socket_stream");
-    }
-					/* check for input in buffer */
-    if ( Spending(s) > 0 )
-    { if ( !PL_unify_list(available, ahead, available) ||
-	   !PL_unify(ahead, head) )
-	return FALSE;
-      from_buffer++;
-    }
-
-    e         = alloca(sizeof(*e));
-    e->fd     = fd;
-    e->stream = PL_copy_term_ref(head);
-    e->next   = map;
-    map       = e;
-
-#ifdef __WINDOWS__
-    if ( ++count > FD_SETSIZE )
-#else
-    if ( fd >= FD_SETSIZE )
-#endif
-      return PL_representation_error("FD_SETSIZE");
-    FD_SET((SOCKET)fd, &fds);
-
-    if ( fd > max )
-      max = fd;
-    if( fd < min )
-      min = fd;
-  }
-  if ( !PL_get_nil(streams) )
-    return pl_error("tcp_select", 3, NULL, ERR_TYPE, Streams, "list");
-
-  if ( from_buffer > 0 )
-    return PL_unify_nil(available);
-
-  if ( PL_get_atom(timeout, &a) && a == ATOM_infinite )
-  { to = NULL;
-  } else
-  { if ( !PL_get_float(timeout, &time) )
-      return pl_error("tcp_select", 3, NULL,
-		      ERR_TYPE, timeout, "number");
-
-    if ( time >= 0.0 )
-    { t.tv_sec  = (int)time;
-      t.tv_usec = ((int)(time * 1000000) % 1000000);
-    } else
-    { t.tv_sec  = 0;
-      t.tv_usec = 0;
-    }
-    to = &t;
-  }
-
-  while( (ret=nbio_select(max+1, &fds, NULL, NULL, to)) == -1 &&
-	 errno == EINTR )
-  { fdentry *e;
-
-    if ( PL_handle_signals() < 0 )
-      return FALSE;			/* exception */
-
-    FD_ZERO(&fds);			/* EINTR may leave fds undefined */
-    for(e=map; e; e=e->next)		/* so we rebuild it to be safe */
-    { FD_SET((SOCKET)e->fd, &fds);
-    }
-  }
-
-  switch(ret)
-  { case -1:
-      return pl_error("tcp_select", 3, NULL, ERR_ERRNO, GET_ERRNO,
-		      "select", "streams", Streams);
-
-    case 0: /* Timeout */
-      break;
-
-    default: /* Something happened -> check fds */
-    { int n;
-
-      for(n=min; n <= max; n++)
-      { if ( FD_ISSET(n, &fds) )
-	{ if ( !PL_unify_list(available, ahead, available) ||
-	       !PL_unify(ahead, findmap(map, n)) )
-	    return FALSE;
-	}
-      }
-      break;
-    }
-  }
-
-  return PL_unify_nil(available);
-}
-#endif /*__WINDOWS__*/
-
-
 #ifdef O_DEBUG
 static foreign_t
 pl_debug(term_t val)
@@ -797,6 +698,7 @@ install_socket(void)
   ATOM_file_no		  = PL_new_atom("file_no");
   ATOM_ip_add_membership  = PL_new_atom("ip_add_membership");
   ATOM_ip_drop_membership = PL_new_atom("ip_drop_membership");
+  ATOM_sndbuf             = PL_new_atom("sndbuf");
 
   FUNCTOR_socket1 = PL_new_functor(PL_new_atom("$socket"), 1);
 
@@ -811,9 +713,6 @@ install_socket(void)
   PL_register_foreign("tcp_getopt",           2, pl_getopt,           0);
   PL_register_foreign("tcp_host_to_address",  2, pl_host_to_address,  0);
   PL_register_foreign("gethostname",          1, pl_gethostname,      0);
-#ifdef __WINDOWS__
-  PL_register_foreign("tcp_select",           3, tcp_select,          0);
-#endif
 
   PL_register_foreign("udp_socket",           1, udp_socket,          0);
   PL_register_foreign("udp_receive",	      4, udp_receive,	      0);
