@@ -4,7 +4,7 @@
     E-mail:        jeffrose@acm.org
     WWW:           http://www.swi-prolog.org
     Copyright (c)  2012-2013, Jeffrey Rosenwald
-		   2018, CWI Amsterdam
+		   2018-2020, CWI Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -41,12 +41,24 @@
             udp_peer_del/2,                     % +Scope, ?IP
             udp_peer/2                          % +Scope, -IP
           ]).
-:- use_module(library(socket)).
-:- use_module(library(broadcast)).
-:- use_module(library(option)).
-:- use_module(library(apply)).
-:- use_module(library(debug)).
-:- use_module(library(error)).
+:- autoload(library(apply),[maplist/2,maplist/3]).
+:- autoload(library(backcomp),[thread_at_exit/1]).
+:- autoload(library(broadcast),
+	    [broadcast_request/1,broadcast/1,listening/3,listen/3]).
+:- autoload(library(debug),[debug/3]).
+:- autoload(library(error),
+	    [must_be/2,syntax_error/1,domain_error/2,existence_error/2]).
+:- autoload(library(option),[option/3]).
+:- autoload(library(socket),
+	    [ tcp_close_socket/1,
+	      udp_socket/1,
+	      tcp_bind/2,
+	      tcp_getopt/2,
+	      tcp_setopt/2,
+	      udp_receive/4,
+	      udp_send/4
+	    ]).
+
 
 % :- debug(udp(broadcast)).
 
@@ -341,6 +353,10 @@ udp_broadcast_address(IPAddress, Subnet, BroadcastAddress) :-
     udp_closed/1.				% Scope
 
 udp_inbound_proxy :-
+    thread_at_exit(inbound_proxy_died),
+    udp_inbound_proxy_loop.
+
+udp_inbound_proxy_loop :-
     make_private_socket,
     forall(udp_scope(Scope, ScopeData),
            make_public_socket(ScopeData, Scope)),
@@ -348,7 +364,7 @@ udp_inbound_proxy :-
     findall(FileNo, udp_socket_file_no(FileNo), FileNos),
     catch(dispatch_inbound(FileNos),
           E, dispatch_exception(E)),
-    udp_inbound_proxy.
+    udp_inbound_proxy_loop.
 
 dispatch_exception(E) :-
     E = error(_,_),
@@ -559,6 +575,29 @@ reload_inbound_proxy :-
                     detached(true)
                   ]).
 
+inbound_proxy_died :-
+    thread_self(Self),
+    thread_property(Self, status(Status)),
+    (   catch(recreate_proxy(Status), _, fail)
+    ->  print_message(informational,
+                      httpd_restarted_worker(Self))
+    ;   done_status_message_level(Status, Level),
+        print_message(Level,
+                      httpd_stopped_worker(Self, Status))
+    ).
+
+recreate_proxy(exception(Error)) :-
+    recreate_on_error(Error),
+    reload_inbound_proxy.
+
+recreate_on_error('$aborted').
+recreate_on_error(time_limit_exceeded).
+
+done_status_message_level(true, silent) :- !.
+done_status_message_level(exception('$aborted'), silent) :- !.
+done_status_message_level(_, informational).
+
+
 %!  udp_broadcast_close(+Scope)
 %
 %   Close a UDP broadcast scope.
