@@ -13,7 +13,7 @@ else
   ${CMAKE_COMMAND} -DPROFILE_GUIDED_OPTIMIZATION=GENERATE -G Ninja ${SRC_DIR}
   ninja -t clean libswipl
   ${CMAKE_COMMAND} -E echo "PGO: Compiling instrumented version"
-  ninja prolog_products
+  ninja core
   ${CMAKE_COMMAND} -E echo "PGO: Running program"
   ${CMAKE_COMMAND} -E remove_directory ${PGO_DIR}
 
@@ -30,6 +30,6 @@ else
   ninja pgo_data
   ninja -t clean libswipl
   ${CMAKE_COMMAND} -E echo "PGO: Compiling optimized core"
-  ninja prolog_products
+  ninja core
   ${CMAKE_COMMAND} -E echo "PGO: run ninja to complete the build"
 fi
