@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2019, University of Amsterdam
+    Copyright (c)  2011-2020, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
     All rights reserved.
@@ -41,6 +41,7 @@
 #include <process.h>			/* getpid() */
 #endif
 #include "pl-incl.h"
+#include "pl-tabling.h"
 #include "pl-ctype.h"
 #include <ctype.h>
 #ifdef HAVE_SYS_TIME_H
@@ -335,6 +336,31 @@ setBackQuotes(atom_t a, unsigned int *flagp)
 }
 
 
+int
+setRationalSyntax(atom_t a, unsigned int *flagp)
+{ GET_LD
+  unsigned int flags;
+
+  if	  ( a == ATOM_natural )
+    flags = RAT_NATURAL;
+  else if ( a == ATOM_compatibility )
+    flags = RAT_COMPAT;
+  else
+  { term_t value = PL_new_term_ref();
+
+    PL_put_atom(value, a);
+    return PL_error(NULL, 0, NULL, ERR_DOMAIN,
+		    ATOM_rational_syntax, value);
+  }
+
+  *flagp &= ~RAT_MASK;
+  *flagp |= flags;
+
+  succeed;
+}
+
+
+
 static int
 setUnknown(term_t value, atom_t a, Module m)
 { unsigned int flags = m->flags & ~(UNKNOWN_MASK);
@@ -515,6 +541,45 @@ setStreamTypeCheck(atom_t a)
   return TRUE;
 }
 
+
+static int
+setAutoload(atom_t a)
+{ GET_LD
+
+  if ( a == ATOM_false )
+    clearPrologFlagMask(PLFLAG_AUTOLOAD);
+  else if ( a == ATOM_explicit ||
+	    a == ATOM_true ||
+	    a == ATOM_user ||
+	    a == ATOM_user_or_explicit )
+    setPrologFlagMask(PLFLAG_AUTOLOAD);
+  else
+  { term_t value = PL_new_term_ref();
+
+    PL_put_atom(value, a);
+    return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_autoload, value);
+  }
+
+  return TRUE;
+}
+
+
+static int
+propagateAutoload(term_t val ARG_LD)
+{ if ( !GD->bootsession )
+  { predicate_t pred;
+    term_t av;
+
+    pred = PL_predicate("set_autoload", 1, "$autoload");
+    return ( (av=PL_new_term_refs(2)) &&
+	     PL_put_term(av+0, val) &&
+	     PL_call_predicate(NULL, PL_Q_PASS_EXCEPTION, pred, av) );
+  } else
+  { return TRUE;
+  }
+}
+
+
 #if O_XOS
 typedef struct access_id
 { char *name;
@@ -560,18 +625,10 @@ get_win_file_access_check(void)
 #endif
 
 static word
-set_prolog_flag_unlocked(term_t key, term_t value, int flags)
-{ GET_LD
-  atom_t k;
-  prolog_flag *f;
-  Module m = MODULE_parse;
+set_prolog_flag_unlocked(Module m, atom_t k, term_t value, int flags ARG_LD)
+{ prolog_flag *f;
   int rval = TRUE;
 
-  if ( !PL_strip_module(key, &m, key) )
-    return FALSE;
-  if ( !PL_get_atom(key, &k) )
-    return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_atom, key);
-
 					/* set existing Prolog flag */
 #ifdef O_PLMT
   if ( LD->prolog_flag.table &&
@@ -584,8 +641,18 @@ set_prolog_flag_unlocked(term_t key, term_t value, int flags)
   { if ( flags & FF_KEEP )
       return TRUE;
     if ( (f->flags&FF_READONLY) && !(flags&FF_FORCE) )
-      return PL_error(NULL, 0, NULL, ERR_PERMISSION,
-		      ATOM_modify, ATOM_flag, key);
+    { term_t key;
+
+      return ( (key = PL_new_term_ref()) &&
+	       PL_put_atom(key, k) &&
+	       PL_error(NULL, 0, NULL, ERR_PERMISSION,
+			ATOM_modify, ATOM_flag, key) );
+    }
+
+    if ( tbl_is_restraint_flag(k) )
+      return tbl_set_restraint_flag(value, k PASS_LD);
+    if ( is_arith_flag(k) )
+      return set_arith_flag(value, k PASS_LD);
 
 #ifdef O_PLMT
     if ( GD->statistics.threads_created > 1 )
@@ -697,9 +764,13 @@ set_prolog_flag_unlocked(term_t key, term_t value, int flags)
 
     if ( PL_current_prolog_flag(ATOM_user_flags, PL_ATOM, &how) )
     { if ( how == ATOM_error )
-	return PL_error(NULL, 0, NULL, ERR_EXISTENCE,
-			ATOM_prolog_flag, key);
-      else if ( how == ATOM_warning )
+      { term_t key;
+
+	return ( (key = PL_new_term_ref()) &&
+		 PL_put_atom(key, k) &&
+		 PL_error(NULL, 0, NULL, ERR_EXISTENCE,
+			  ATOM_prolog_flag, key) );
+      } else if ( how == ATOM_warning )
 	Sdprintf("WARNING: Flag %s: new Prolog flags must be created using "
 		 "create_prolog_flag/3\n", stringAtom(k));
     }
@@ -788,6 +859,8 @@ set_prolog_flag_unlocked(term_t key, term_t value, int flags)
       { rval = setDoubleQuotes(a, &m->flags);
       } else if ( k == ATOM_back_quotes )
       { rval = setBackQuotes(a, &m->flags);
+      } else if ( k == ATOM_rational_syntax )
+      { rval = setRationalSyntax(a, &m->flags);
       } else if ( k == ATOM_unknown )
       { rval = setUnknown(value, a, m);
       } else if ( k == ATOM_write_attributes )
@@ -802,6 +875,8 @@ set_prolog_flag_unlocked(term_t key, term_t value, int flags)
       { rval = setStreamTypeCheck(a);
       } else if ( k == ATOM_file_name_case_handling )
       { rval = setFileNameCaseHandling(a);
+      } else if ( k == ATOM_autoload )
+      { rval = setAutoload(a);
 #if O_XOS
       } else if ( k == ATOM_win_file_access_check )
       { rval = set_win_file_access_check(value);
@@ -810,9 +885,11 @@ set_prolog_flag_unlocked(term_t key, term_t value, int flags)
       if ( !rval )
 	fail;
 
-      PL_unregister_atom(f->value.a);
-      f->value.a = a;
-      PL_register_atom(a);
+      if ( f->value.a != a )
+      { PL_unregister_atom(f->value.a);
+	f->value.a = a;
+	PL_register_atom(a);
+      }
       break;
     }
     case FT_INTEGER:
@@ -873,10 +950,20 @@ set_prolog_flag_unlocked(term_t key, term_t value, int flags)
 
 int
 set_prolog_flag(term_t key, term_t value, int flags)
-{ int rc;
+{ GET_LD
+  atom_t k;
+  Module m = MODULE_parse;
+  int rc;
+
+  if ( !PL_strip_module(key, &m, key) ||
+       !PL_get_atom_ex(key, &k) )
+    return FALSE;
+
+  if ( k == ATOM_autoload && !propagateAutoload(value PASS_LD) )
+    return FALSE;
 
   PL_LOCK(L_PLFLAG);
-  rc = set_prolog_flag_unlocked(key, value, flags);
+  rc = set_prolog_flag_unlocked(m, k, value, flags PASS_LD);
   PL_UNLOCK(L_PLFLAG);
 
   return rc;
@@ -1039,6 +1126,16 @@ unify_prolog_flag_value(Module m, atom_t key, prolog_flag *f, term_t val)
     else
       v = ATOM_symbol_char;
 
+    return PL_unify_atom(val, v);
+  } else if ( key == ATOM_rational_syntax )
+  { atom_t v;
+
+    switch(m->flags&RAT_MASK)
+    { case RAT_NATURAL: v = ATOM_natural;       break;
+      case RAT_COMPAT:  v = ATOM_compatibility; break;
+      default:		v = 0; assert(0);
+    }
+
     return PL_unify_atom(val, v);
   } else if ( key == ATOM_unknown )
   { atom_t v;
@@ -1077,6 +1174,10 @@ unify_prolog_flag_value(Module m, atom_t key, prolog_flag *f, term_t val)
   { return PL_unify_atom(val, accessLevel());
   } else if ( key == ATOM_stack_limit )
   { return PL_unify_int64(val, LD->stacks.limit);
+  } else if ( tbl_is_restraint_flag(key) )
+  { return tbl_get_restraint_flag(val, key PASS_LD) == TRUE;
+  } else if ( is_arith_flag(key) )
+  { return get_arith_flag(val, key PASS_LD) == TRUE;
   }
 
   switch(f->flags & FT_MASK)
@@ -1401,7 +1502,8 @@ initPrologFlags(void)
   setPrologFlag("user_flags", FT_ATOM, "silent");
   setPrologFlag("editor", FT_ATOM, "default");
   setPrologFlag("debugger_show_context", FT_BOOL, FALSE, 0);
-  setPrologFlag("autoload",  FT_BOOL, TRUE,  PLFLAG_AUTOLOAD);
+  setPrologFlag("autoload",  FT_ATOM, "true");
+  setPrologFlagMask(PLFLAG_AUTOLOAD);
 #ifndef O_GMP
   setPrologFlag("max_integer",	   FT_INT64|FF_READONLY, PLMAXINT);
   setPrologFlag("min_integer",	   FT_INT64|FF_READONLY, PLMININT);
@@ -1409,7 +1511,11 @@ initPrologFlags(void)
   setPrologFlag("max_tagged_integer", FT_INTEGER|FF_READONLY, PLMAXTAGGEDINT);
   setPrologFlag("min_tagged_integer", FT_INTEGER|FF_READONLY, PLMINTAGGEDINT);
 #ifdef O_GMP
-  setPrologFlag("bounded",		   FT_BOOL|FF_READONLY,	   FALSE, 0);
+  setPrologFlag("bounded",	      FT_BOOL|FF_READONLY,	   FALSE, 0);
+  setPrologFlag("prefer_rationals", FT_BOOL, O_PREFER_RATIONALS, PLFLAG_RATIONAL);
+  setPrologFlag("rational_syntax",  FT_ATOM,
+		O_RATIONAL_SYNTAX == RAT_NATURAL ? "natural" :
+		                                   "compatibility");
 #ifdef __GNU_MP__
   setPrologFlag("gmp_version",	   FT_INTEGER|FF_READONLY, __GNU_MP__);
 #endif
