@@ -107,15 +107,8 @@ COMMON(int)		checkAtoms_src(const char *file, int line);
 COMMON(int)		is_volatile_atom(atom_t a);
 #ifdef O_DEBUG_ATOMGC
 COMMON(word)		pl_track_atom(term_t which, term_t stream);
-COMMON(void)		_PL_debug_register_atom(atom_t a,
-					const char *file, int line,
-					const char *func);
-COMMON(void)		_PL_debug_unregister_atom(atom_t a,
-					  const char *file, int line,
-					  const char *func);
 #endif
 
-
 /* pl-arith.c */
 
 COMMON(int)		ar_compare(Number n1, Number n2, int what);
@@ -137,12 +130,15 @@ COMMON(Number)		growArithStack(ARG1_LD);
 COMMON(void)		freeArithLocalData(PL_local_data_t *ld);
 COMMON(int)		ar_sign_i(Number n1);
 COMMON(int)		ar_signbit(Number n1);
-COMMON(int)		check_float(double f);
+COMMON(int)		check_float(Number n);
 COMMON(int)		ar_rdiv_mpz(Number n1, Number n2, Number r);
 COMMON(int)		PL_eval_expression_to_int64_ex(term_t t, int64_t *val);
 COMMON(int)		is_arith_flag(atom_t k);
 COMMON(int)		get_arith_flag(term_t val, atom_t k ARG_LD);
 COMMON(int)		set_arith_flag(term_t val, atom_t k ARG_LD);
+COMMON(void)		set_rounding(int mode);
+COMMON(int)		atom_to_rounding(atom_t a, int *m);
+COMMON(atom_t)		float_rounding_name(int m);
 
 /* pl-bag.c */
 COMMON(void)		markAtomsFindall(PL_local_data_t *ld);
