@@ -35,6 +35,8 @@
 */
 
 /*#define O_DEBUG 1*/
+#include <math.h>
+#include <fenv.h>
 #include "pl-incl.h"
 #include "pl-inline.h"
 #undef LD
@@ -770,6 +772,11 @@ promoteToMPQNumber(number *n)
     case V_FLOAT:
     { double v = n->value.f;
 
+      if ( isnan(v) )
+	return PL_error(NULL, 0, NULL, ERR_AR_UNDEF);
+      if ( isinf(v) )
+	return PL_error(NULL, 0, NULL, ERR_AR_OVERFLOW);
+
       n->type = V_MPQ;
       mpq_init(n->value.mpq);
       mpq_set_d(n->value.mpq, v);
@@ -1201,6 +1208,23 @@ PL_get_number(term_t t, Number n)
 		 *	     PROMOTION		*
 		 *******************************/
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+The GMP functions which convert  mpz's   and  mpq's to double's truncate
+(round to zero) if necessary ignoring the current IEEE rounding mode. To
+correct this incorrect  rounding  when  the   mode  is  `to_postive`  or
+`to_negative` all calls  to  mpX_get_d()  should   be  wrapped  in  this
+function. Note that this function has no GMP dependencies.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+double
+mpX_round(double d) {
+  switch (fegetround()) {
+    case FE_UPWARD  : return (d>=0) ? nexttoward(d, INFINITY) : d;
+    case FE_DOWNWARD: return (d<=0) ? nexttoward(d,-INFINITY) : d;
+    default: return d;
+  }
+}
+
 int
 promoteToFloatNumber(Number n)
 { switch(n->type)
@@ -1210,10 +1234,7 @@ promoteToFloatNumber(Number n)
       break;
 #ifdef O_GMP
     case V_MPZ:
-    { double val = mpz_get_d(n->value.mpz);
-
-      if ( !check_float(val) )
-	return FALSE;
+    { double val = mpX_round(mpz_get_d(n->value.mpz));
 
       clearNumber(n);
       n->value.f = val;
@@ -1221,10 +1242,7 @@ promoteToFloatNumber(Number n)
       break;
     }
     case V_MPQ:
-    { double val = mpq_get_d(n->value.mpq);
-
-      if ( !check_float(val) )
-	return FALSE;
+    { double val = mpq_to_double(n->value.mpq);
 
       clearNumber(n);
       n->value.f = val;
@@ -1236,7 +1254,7 @@ promoteToFloatNumber(Number n)
       break;
   }
 
-  return TRUE;
+  return check_float(n);
 }
 
 
@@ -1285,8 +1303,8 @@ cmpNumbers() compares two numbers. First, both   numbers are promoted to
 the lowest (in V_* ordering) common type, after which they are compared.
 Note that if the common type is V_FLOAT, but not both are V_FLOAT we can
 run into troubles because big  integers  may   be  out  of range for the
-double representation. We trust mpz_get_d()   and mpq_get_d() return +/-
-float infinity and this compares  correctly   with  the  other argument,
+double representation. We trust mpz_get_d()   and mpq_to_double() return
++/- float infinity and this compares  correctly with the other argument,
 which is guaranteed to be a valid float.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
@@ -1295,16 +1313,19 @@ cmpFloatNumbers(Number n1, Number n2)
 { if ( n1->type == V_FLOAT )
   { double d2;
 
+    if ( isnan(n1->value.f) )
+      return CMP_NOTEQ;
+
     switch(n2->type)
     { case V_INTEGER:
 	d2 = (double)n2->value.i;
 	break;
 #ifdef O_GMP
       case V_MPZ:
-	d2 = mpz_get_d(n2->value.mpz);
+	d2 = mpX_round(mpz_get_d(n2->value.mpz));
 	break;
       case V_MPQ:
-	d2 = mpq_get_d(n2->value.mpq);
+	d2 = mpq_to_double(n2->value.mpq);
 	break;
 #endif
       default:
@@ -1319,16 +1340,19 @@ cmpFloatNumbers(Number n1, Number n2)
 
     assert(n2->type == V_FLOAT);
 
+    if ( isnan(n2->value.f) )
+      return CMP_NOTEQ;
+
     switch(n1->type)
     { case V_INTEGER:
 	d1 = (double)n1->value.i;
 	break;
 #ifdef O_GMP
       case V_MPZ:
-	d1 = mpz_get_d(n1->value.mpz);
+	d1 = mpX_round(mpz_get_d(n1->value.mpz));
 	break;
       case V_MPQ:
-	d1 = mpq_get_d(n1->value.mpq);
+	d1 = mpq_to_double(n1->value.mpq);
 	break;
 #endif
       default:
@@ -1379,7 +1403,6 @@ cmpNumbers(Number n1, Number n2)
   return CMP_EQUAL;
 }
 
-
 void
 cpNumber(Number to, Number from)
 { to->type = from->type;
@@ -1403,13 +1426,170 @@ cpNumber(Number to, Number from)
   }
 }
 
+#ifdef O_GMP
+
+		 /*******************************
+		 *	 FLOAT <-> RATIONAL	*
+		 *******************************/
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+This code is copied from ECLiPSe  7.0_53.   This  code is covered by the
+CMPL 1.1 (Cisco-style Mozilla Public License  Version 1.1), available at
+www.eclipse-clp.org/license.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+/*
+ * Divide two bignums giving a double float result. The naive solution
+ *	return mpz_to_double(num) / mpz_to_double(den);
+ * suffers from floating point overflows when the numbers are huge
+ * and is inefficient because it looks at unnecessarily many digits.
+ *
+ * IEEE double precision is 53 bits mantissa and 12 bits signed exponent.
+ * So the largest integer representable with doubles is 1024 bits wide,
+ * of which the first 53 are ones, i.e. it lies between 2^1023 and 2^1024.
+ * If the dividend's MSB is more than 1024 bits higher than the divisor's,
+ * the result will always be floating point infinity (no need to divide).
+ * If we do divide, we first drop excess integer precision by keeping only
+ * DBL_PRECISION_LIMBS and ignoring the lower limbs for both operands
+ * (i.e. we effectively scale the integers down, or right-shift them).
+ */
+
+#define MIN_LIMB_DIFF (1+1024/GMP_NUMB_BITS)
+#define DBL_PRECISION_LIMBS (2+53/GMP_NUMB_BITS)
+#define MAX_ULONG_DBL ((double)~0L+1.0)
+
+static double
+mpz_fdiv(mpz_t num, mpz_t den)
+{ mp_ptr longer_d, shorter_d;
+  mp_size_t shorter_size, longer_size, ignored_limbs = 0;
+  int negative, swapped;
+  /* By declaring res volatile we make sure that the result is rounded
+   * to double precision instead of being returned with extended precision
+   * in a floating point register, which can have confusing consequences */
+  volatile double res;
+
+  shorter_size = num->_mp_size;
+  longer_size = den->_mp_size;
+  negative = 0;
+
+  if ( shorter_size < 0 )
+  { shorter_size = -shorter_size;
+    negative = !negative;
+  }
+  if ( longer_size < 0 )
+  { longer_size = -longer_size;
+    negative = !negative;
+  }
+  if ( shorter_size > longer_size )
+  { longer_size = shorter_size;
+    longer_d = num->_mp_d;
+    shorter_size = (den->_mp_size >= 0 ? den->_mp_size : -den->_mp_size);
+    shorter_d = den->_mp_d;
+    swapped = 1;			/* abs(res) > 1 */
+  } else
+  { longer_d = den->_mp_d;
+    shorter_d = num->_mp_d;
+    swapped = 0;			/* abs(res) < 1 */
+  }
+
+  if ( longer_size - shorter_size > MIN_LIMB_DIFF )
+  { res = swapped ? HUGE_VAL : 0.0;
+  } else
+  { double l,s;
+    long int le, se;
+    mpz_t li, si;
+
+    /* we ignore limbs that are not significant for the result */
+    if ( longer_size > MIN_LIMB_DIFF )	/* more can't be represented */
+    { ignored_limbs = longer_size - MIN_LIMB_DIFF;
+      longer_size -= ignored_limbs;
+      shorter_size -= ignored_limbs;
+    }
+    if ( shorter_size > DBL_PRECISION_LIMBS )	/* more exceeds the precision */
+    { ignored_limbs += shorter_size - DBL_PRECISION_LIMBS;
+      longer_size -= shorter_size - DBL_PRECISION_LIMBS;
+      shorter_size = DBL_PRECISION_LIMBS;
+    }
+    longer_d += ignored_limbs;
+    shorter_d += ignored_limbs;
+    li->_mp_alloc = li->_mp_size = longer_size; li->_mp_d = longer_d;
+    si->_mp_alloc = si->_mp_size = shorter_size; si->_mp_d = shorter_d;
+
+    l = mpz_get_d_2exp(&le, li);
+    s = mpz_get_d_2exp(&se, si);
+    if ( swapped )
+      res = (l/s) * pow(2.0, le-se);
+    else
+      res = (s/l) * pow(2.0, se-le);
+  }
+
+  return negative ? -res : res;
+}
+
+double
+mpq_to_double(mpq_t q)
+{ return mpz_fdiv(mpq_numref(q), mpq_denref(q));
+}
+
+void
+mpq_set_double(mpq_t q, double f)
+{ double fabs = (f < 0.0) ? -f : f;	/* get rid of the sign */
+  double x = fabs;
+  mpz_t na, nb, da, db, big_xi, tmpn, tmpd;
+
+  mpz_init_set_ui(na, 1L);
+  mpz_init_set_ui(nb, 0L);
+  mpz_init_set_ui(da, 0L);
+  mpz_init_set_ui(db, 1L);
+  mpz_init(tmpn);
+  mpz_init(tmpd);
+  mpz_init(big_xi);
+
+  while ( mpz_fdiv(na, da) != fabs )
+  { double xi = floor(x);
+    double xf = x - xi;
+
+    mpz_swap(tmpn, na);
+    mpz_swap(tmpd, da);
+
+    if ( x < MAX_ULONG_DBL )
+    { unsigned long int_xi = (unsigned long) xi;
+      mpz_mul_ui(na, tmpn, int_xi);
+      mpz_mul_ui(da, tmpd, int_xi);
+    } else
+    { mpz_set_d(big_xi, xi);
+      mpz_mul(na, tmpn, big_xi);
+      mpz_mul(da, tmpd, big_xi);
+    }
+    mpz_add(na, na, nb);
+    mpz_add(da, da, db);
+    mpz_swap(nb, tmpn);
+    mpz_swap(db, tmpd);
+
+    if ( xf == 0.0 )
+      break;
+    x = 1.0/xf;
+  }
+
+  if (f < 0.0)			/* compute q := [+-]na/da */
+      mpz_neg(na, na);
+  mpq_set_num(q, na);
+  mpq_set_den(q, da);
+  mpq_canonicalize(q);
+
+  mpz_clear(big_xi);			/* clean up */
+  mpz_clear(tmpd);
+  mpz_clear(tmpn);
+  mpz_clear(db);
+  mpz_clear(da);
+  mpz_clear(nb);
+  mpz_clear(na);
+}
 
 		 /*******************************
 		 *	 PUBLIC INTERFACE	*
 		 *******************************/
 
-#ifdef O_GMP
-
 int
 PL_get_mpz(term_t t, mpz_t mpz)
 { GET_LD
