@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2019, University of Amsterdam,
+    Copyright (c)  1985-2020, University of Amsterdam,
                               VU University Amsterdam
 			      CWI, Amsterdam
     All rights reserved.
@@ -144,6 +144,10 @@ handy for it someone wants to add a data type to the system.
       Use GNU gmp library for infinite precision arthmetic
   O_MITIGATE_SPECTRE
       Reduce spectre security risc.  Currently reduces timer resolution.
+  O_PREFER_RATIONALS
+      Default for the `prefer_rationals` flag.
+  O_RATIONAL_SYNTAX
+      Default support for rational syntax (RAT_NATURAL or RAT_COMPAT)
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #define O_COMPILE_OR		1
@@ -171,6 +175,12 @@ handy for it someone wants to add a data type to the system.
 #define O_GVAR			1
 #define O_CYCLIC		1
 #define O_MITIGATE_SPECTRE	1
+#ifndef O_PREFER_RATIONALS
+#define O_PREFER_RATIONALS	FALSE
+#endif
+#ifndef O_RATIONAL_SYNTAX
+#define O_RATIONAL_SYNTAX	RAT_COMPAT
+#endif
 
 #if defined(O_PLMT)
 #if defined(O_SIGPROF_PROFILE) || defined(__WINDOWS__)
@@ -742,8 +752,10 @@ typedef struct
 
 #ifdef O_GMP
 #define intNumber(n)	((n)->type <=  V_MPZ)
+#define ratNumber(n)	((n)->type <=  V_MPQ)
 #else
 #define intNumber(n)	((n)->type <  V_FLOAT)
+#define ratNumber(n)	((n)->type <  V_FLOAT)
 #endif
 #define floatNumber(n)	((n)->type >= V_FLOAT)
 
@@ -862,7 +874,7 @@ with one operation, it turns out to be faster as well.
 #define SPY_ME			(0x01000000) /* Spy point placed */
 #define TRACE_ME		(0x02000000) /* Can be debugged */
 #define P_INCREMENTAL		(0x04000000) /* Incremental tabling */
-#define P_ABSTRACT		(0x08000000) /* Incremental tabling */
+#define P_AUTOLOAD		(0x08000000) /* autoload/2 explicit import */
 #define P_TSHARED		(0x10000000) /* Using a shared table */
 #define	P_LOCKED_SUPERVISOR	(0x20000000) /* Fixed supervisor */
 #define FILE_ASSIGNED		(0x40000000) /* Is assigned to a file */
@@ -890,22 +902,25 @@ with one operation, it turns out to be faster as well.
    in pl-read.c.
 */
 
-#define M_SYSTEM		(0x0001) /* system module */
-#define M_CHARESCAPE		(0x0002) /* module */
-#define DBLQ_CHARS		(0x0004) /* "ab" --> ['a', 'b'] */
-#define DBLQ_ATOM		(0x0008) /* "ab" --> 'ab' */
-#define DBLQ_STRING		(0x0010) /* "ab" --> "ab" */
+#define M_SYSTEM		(0x00000001) /* system module */
+#define M_CHARESCAPE		(0x00000002) /* module */
+#define DBLQ_CHARS		(0x00000004) /* "ab" --> ['a', 'b'] */
+#define DBLQ_ATOM		(0x00000008) /* "ab" --> 'ab' */
+#define DBLQ_STRING		(0x00000010) /* "ab" --> "ab" */
 #define DBLQ_MASK		(DBLQ_CHARS|DBLQ_ATOM|DBLQ_STRING)
-#define BQ_STRING		(0x0020) /* `ab` --> "ab" */
-#define BQ_CODES		(0x0040) /* `ab` --> [97,98] */
-#define BQ_CHARS		(0x0080) /* `ab` --> [a,b] */
+#define BQ_STRING		(0x00000020) /* `ab` --> "ab" */
+#define BQ_CODES		(0x00000040) /* `ab` --> [97,98] */
+#define BQ_CHARS		(0x00000080) /* `ab` --> [a,b] */
 #define BQ_MASK			(BQ_STRING|BQ_CODES|BQ_CHARS)
-#define UNKNOWN_FAIL		(0x0100) /* module */
-#define UNKNOWN_WARNING		(0x0200) /* module */
-#define UNKNOWN_ERROR		(0x0400) /* module */
+#define RAT_COMPAT		(0)
+#define RAT_NATURAL		(0x00000100) /* 1/3 */
+#define RAT_MASK		(RAT_NATURAL)
+#define UNKNOWN_FAIL		(0x00001000) /* module */
+#define UNKNOWN_WARNING		(0x00002000) /* module */
+#define UNKNOWN_ERROR		(0x00004000) /* module */
 #define UNKNOWN_MASK		(UNKNOWN_ERROR|UNKNOWN_WARNING|UNKNOWN_FAIL)
-#define M_VARPREFIX		(0x0800)
-#define M_DESTROYED		(0x1000)
+#define M_VARPREFIX		(0x00008000) /* _var, Atom */
+#define M_DESTROYED		(0x00010000)
 
 /* Flags on functors */
 
@@ -1361,15 +1376,16 @@ typedef struct gc_stats
 #define CA1_FLOAT	6	/* next WORDS_PER_DOUBLE are double */
 #define CA1_STRING	7	/* inlined string */
 #define CA1_MPZ	        8	/* GNU mpz number */
-#define CA1_MODULE      9	/* a module */
-#define CA1_VAR	       10	/* a variable(-offset) */
-#define CA1_FVAR       11	/* a variable(-offset), used as `firstvar' */
-#define CA1_CHP	       12	/* ChoicePoint (also variable(-offset)) */
-#define CA1_FOREIGN    13	/* Foreign function pointer */
-#define CA1_CLAUSEREF  14	/* Clause reference */
-#define CA1_JUMP       15	/* Instructions to skip */
-#define CA1_AFUNC      16	/* Number of arithmetic function */
-#define CA1_TRIE_NODE  17	/* Tabling: answer trie node with delays */
+#define CA1_MPQ	        9	/* GNU mpq number */
+#define CA1_MODULE     10	/* a module */
+#define CA1_VAR	       11	/* a variable(-offset) */
+#define CA1_FVAR       12	/* a variable(-offset), used as `firstvar' */
+#define CA1_CHP	       13	/* ChoicePoint (also variable(-offset)) */
+#define CA1_FOREIGN    14	/* Foreign function pointer */
+#define CA1_CLAUSEREF  15	/* Clause reference */
+#define CA1_JUMP       16	/* Instructions to skip */
+#define CA1_AFUNC      17	/* Number of arithmetic function */
+#define CA1_TRIE_NODE  18	/* Tabling: answer trie node with delays */
 
 #define VIF_BREAK      0x01	/* Can be a breakpoint */
 
@@ -1455,7 +1471,8 @@ struct definition
   unsigned int  shared;			/* #procedures sharing this def */
   struct linger_list  *lingering;	/* Assocated lingering objects */
   gen_t		last_modified;		/* Generation I was last modified */
-  struct event_list *events;		/* Forward update events */
+  struct event_list  *events;		/* Forward update events */
+  struct table_props *tabling;		/* Extended properties for tabling */
 #ifdef O_PROF_PENTIUM
   int		prof_index;		/* index in profiling */
   char	       *prof_name;		/* name in profiling */
@@ -1465,7 +1482,7 @@ struct definition
 struct definition_chain
 { Definition		definition;	/* chain on definition */
   DefinitionChain	next;		/* next in chain */
-};
+} definition_chain;
 
 struct dirty_def_info
 { gen_t		oldest_generation;	/* Oldest generation seen */
@@ -1484,6 +1501,7 @@ typedef struct definition_refs
 
 #define	PROC_WEAK	 (0x0001)	/* implicit import */
 #define	PROC_MULTISOURCE (0x0002)	/* Assigned to multiple sources */
+#define PROC_IMPORTED	 (0x0004)	/* Procedure is imported */
 
 struct procedure
 { Definition	 definition;		/* definition of procedure */
@@ -2373,6 +2391,7 @@ typedef struct debuginfo
 #define PLFLAG_MITIGATE_SPECTRE	    0x04000000 /* Mitigate spectre attacks */
 #define PLFLAG_TABLE_INCREMENTAL    0x08000000 /* By default incremental tabling */
 #define PLFLAG_TABLE_SHARED	    0x10000000 /* By default shared tabling */
+#define PLFLAG_RATIONAL		    0x20000000 /* Natural rational numbers */
 
 typedef struct
 { unsigned int flags;		/* Fast access to some boolean Prolog flags */
