@@ -38,6 +38,10 @@
 #undef LD
 #define LD LOCAL_LD
 
+#ifdef __WINDOWS__
+#include <windows.h>
+#undef small
+#endif
 
 		 /*******************************
 		 *	 LOCK-FREE SUPPORT	*
@@ -63,10 +67,10 @@ MSB(size_t i)
 { unsigned long index;
 #if SIZEOF_VOIDP == 8
   unsigned __int64 mask = i;
-  _BitScanReverse64(&index, mask);
+  BitScanReverse64(&index, mask);
 #else
   unsigned long mask = i;
-  _BitScanReverse(&index, mask);
+  BitScanReverse(&index, mask);
 #endif
 
   return index;
@@ -76,15 +80,11 @@ MSB(size_t i)
 static inline int
 MSB64(int64_t i)
 { unsigned long index;
-  _BitScanReverse64(&index, i);
+  BitScanReverse64(&index, i);
   return index;
 }
 
-
-#define HAVE_MEMORY_BARRIER 1
-#ifndef MemoryBarrier
-#define MemoryBarrier() (void)0
-#endif
+#define MEMORY_BARRIER() MemoryBarrier()
 
 static inline size_t
 __builtin_popcount(size_t sz)
@@ -110,11 +110,8 @@ __builtin_popcount(size_t sz)
 #define MSB64(i) ((int)sizeof(long long)*8-1-__builtin_clzll(i))
 #endif
 
-#if !defined(HAVE_MEMORY_BARRIER) && defined(HAVE__SYNC_SYNCHRONIZE)
-#define HAVE_MEMORY_BARRIER 1
-#ifndef MemoryBarrier
-#define MemoryBarrier()			__sync_synchronize()
-#endif
+#ifdef HAVE__SYNC_SYNCHRONIZE
+#define MEMORY_BARRIER()		__sync_synchronize()
 #endif
 
 #ifdef O_PLMT
@@ -173,9 +170,8 @@ MSB64(int64_t i)
 #endif
 
 
-#ifndef HAVE_MEMORY_BARRIER
-#define HAVE_MEMORY_BARRIER 1
-#define MemoryBarrier() (void)0
+#ifndef MEMORY_BARRIER
+#define MEMORY_BARRIER() (void)0
 #endif
 
 		 /*******************************
